@@ -18,11 +18,12 @@ from facepy.exceptions import *
 
 
 class GraphAPI(object):
-    def __init__(self, oauth_token=False, url='https://graph.facebook.com', verify_ssl_certificate=True, appsecret=False, timeout=None):
+    def __init__(self, oauth_token=False, url='https://graph.facebook.com', verify_ssl_certificate=True, appsecret=False, timeout=None, version=None):
         """
         Initialize GraphAPI with an OAuth access token.
 
         :param oauth_token: A string describing an OAuth access token.
+        :param version: A string with version ex. '2.2'.
         """
         self.oauth_token = oauth_token
         self.session = requests.session()
@@ -30,9 +31,10 @@ class GraphAPI(object):
         self.verify_ssl_certificate = verify_ssl_certificate
         self.appsecret = appsecret
         self.timeout = timeout
+        self.version = version
 
     @classmethod
-    def for_application(self, id, secret_key):
+    def for_application(self, id, secret_key, api_version=None):
         """
         Initialize GraphAPI with an OAuth access token for an application.
 
@@ -41,7 +43,8 @@ class GraphAPI(object):
         """
         from facepy.utils import get_application_access_token
 
-        return GraphAPI(get_application_access_token(id, secret_key))
+        access_token = get_application_access_token(id, secret_key, api_version=api_version)
+        return GraphAPI(access_token, version=api_version)
 
     def get(self, path='', page=False, retry=3, **options):
         """
@@ -224,11 +227,11 @@ class GraphAPI(object):
 
         if(data):
             data = dict(
-                 (k.replace('_sqbro_', '['), v) for k, v in data.items())
+                (k.replace('_sqbro_', '['), v) for k, v in data.items())
             data = dict(
-                 (k.replace('_sqbrc_', ']'), v) for k, v in data.items())
+                (k.replace('_sqbrc_', ']'), v) for k, v in data.items())
             data = dict(
-                 (k.replace('__', ':'), v) for k, v in data.items())
+                (k.replace('__', ':'), v) for k, v in data.items())
         data = data or {}
 
         def load(method, url, data):
@@ -240,8 +243,10 @@ class GraphAPI(object):
 
             try:
                 if method in ['GET', 'DELETE']:
-                    response = self.session.request(method, url, params=data, allow_redirects=True,
-                     verify=self.verify_ssl_certificate, timeout=self.timeout)
+                    response = self.session.request(
+                        method, url, params=data, allow_redirects=True,
+                        verify=self.verify_ssl_certificate, timeout=self.timeout
+                    )
 
                 if method in ['POST', 'PUT']:
                     files = {}
@@ -253,11 +258,28 @@ class GraphAPI(object):
                     for key in files:
                         data.pop(key)
 
-                    response = self.session.request(method, url, data=data, files=files,
-                        verify=self.verify_ssl_certificate, timeout=self.timeout)
+                    response = self.session.request(
+                        method, url, data=data, files=files,
+                        verify=self.verify_ssl_certificate, timeout=self.timeout
+                    )
+
                 if 500 <= response.status_code < 600:
-                    raise InternalFacebookError("Internal Facebook error ocurred (%s)."
-                                                % response.status_code)
+                    # Facebook 5XX errors usually come with helpful messages
+                    # as a JSON object describing the problem with the request.
+                    # If this is the case, an error will be raised and we just
+                    # need to re-raise it. This is most likely to happen
+                    # with the Ads API.
+                    # This will raise an exception if a JSON-like error object
+                    # comes in the response.
+                    self._parse(response.content)
+                    # If Facebook does not provide any JSON-formatted error
+                    # but just a plain-text, useless error, we'll just inform
+                    # about a Facebook Internal errror occurred.
+                    raise FacebookError(
+                        'Internal Facebook error occurred',
+                        response.status_code
+                    )
+
             except requests.RequestException as exception:
                 raise HTTPError(exception)
 
@@ -293,7 +315,7 @@ class GraphAPI(object):
             else:
                 path = '/' + path
 
-        url = '%s%s' % (self.url, path)
+        url = self._get_url(path)
 
         if self.oauth_token:
             data['access_token'] = self.oauth_token
@@ -312,6 +334,13 @@ class GraphAPI(object):
             else:
                 raise
 
+    def _get_url(self, path):
+        if self.version:
+            url = '%s/v%s%s' % (self.url, self.version, path)
+        else:
+            url = '%s%s' % (self.url, path)
+        return url
+
     def _parse(self, data):
         """
         Parse the response from Facebook's Graph API.
